@@ -1,6 +1,6 @@
 /*
 htop - CRT.c
-(C) 2004,2005 Hisham H. Muhammad
+(C) 2004-2006 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
 */
@@ -18,6 +18,13 @@ in the source distribution for its full text.
 
 #define ColorPair(i,j) COLOR_PAIR((7-i)*8+j)
 
+#define COLORSCHEME_DEFAULT 0
+#define COLORSCHEME_MONOCHROME 1
+#define COLORSCHEME_BLACKONWHITE 2
+#define COLORSCHEME_BLACKONWHITE2 3
+#define COLORSCHEME_MIDNIGHT 4
+#define COLORSCHEME_BLACKNIGHT 5
+
 #define Black COLOR_BLACK
 #define Red COLOR_RED
 #define Green COLOR_GREEN
@@ -89,22 +96,17 @@ typedef enum ColorElements_ {
    LAST_COLORELEMENT
 } ColorElements;
 
-extern int CRT_delay;
-
-extern int CRT_colors[LAST_COLORELEMENT];
-
 }*/
 
 // TODO: centralize these in Settings.
 
-/* private property */
-int CRT_delay; 
+int CRT_delay = 0;
 
-/* private property */
-int CRT_colorScheme;
+int CRT_colorScheme = 0;
 
-/* private property */
-int CRT_colors[LAST_COLORELEMENT];
+int CRT_colors[LAST_COLORELEMENT] = { 0 };
+
+char* CRT_termType;
 
 // TODO: pass an instance of Settings instead.
 
@@ -124,8 +126,8 @@ void CRT_init(int delay, int colorScheme) {
    } else {
       CRT_hasColors = false;
    }
-   char* termType = getenv("TERM");
-   if (String_eq(termType, "xterm") || String_eq(termType, "xterm-color") || String_eq(termType, "vt220")) {
+   CRT_termType = getenv("TERM");
+   if (String_eq(CRT_termType, "xterm") || String_eq(CRT_termType, "xterm-color") || String_eq(CRT_termType, "vt220")) {
       define_key("\033[H", KEY_HOME);
       define_key("\033[F", KEY_END);
       define_key("\033OP", KEY_F(1));
@@ -187,7 +189,7 @@ void CRT_handleSIGTERM(int signal) {
 void CRT_setColors(int colorScheme) {
    CRT_colorScheme = colorScheme;
    if (colorScheme == COLORSCHEME_BLACKNIGHT) {
-      for (int i = 0; i < 8; i++) 
+      for (int i = 0; i < 8; i++)
          for (int j = 0; j < 8; j++)
             init_pair((7-i)*8+j, i, j);
    } else {
