@@ -1,6 +1,6 @@
 /*
 htop
-(C) 2004 Hisham H. Muhammad
+(C) 2004-2006 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
 */
@@ -16,7 +16,6 @@ in the source distribution for its full text.
 typedef struct Hashtable_ Hashtable;
 
 typedef void(*Hashtable_PairFunction)(int, void*, void*);
-typedef int(*Hashtable_HashAlgorithm)(Hashtable*, int);
 
 typedef struct HashtableItem {
    int key;
@@ -28,7 +27,6 @@ struct Hashtable_ {
    int size;
    HashtableItem** buckets;
    int items;
-   Hashtable_HashAlgorithm hashAlgorithm;
    bool owner;
 };
 }*/
@@ -49,15 +47,10 @@ Hashtable* Hashtable_new(int size, bool owner) {
    this = (Hashtable*) malloc(sizeof(Hashtable));
    this->size = size;
    this->buckets = (HashtableItem**) calloc(sizeof(HashtableItem*), size);
-   this->hashAlgorithm = Hashtable_hashAlgorithm;
    this->owner = owner;
    return this;
 }
 
-int Hashtable_hashAlgorithm(Hashtable* this, int key) {
-   return (key % this->size);
-}
-
 void Hashtable_delete(Hashtable* this) {
    for (int i = 0; i < this->size; i++) {
       HashtableItem* walk = this->buckets[i];
@@ -78,7 +71,7 @@ inline int Hashtable_size(Hashtable* this) {
 }
 
 void Hashtable_put(Hashtable* this, int key, void* value) {
-   int index = this->hashAlgorithm(this, key);
+   int index = key % this->size;
    HashtableItem** bucketPtr = &(this->buckets[index]);
    while (true)
       if (*bucketPtr == NULL) {
@@ -95,7 +88,7 @@ void Hashtable_put(Hashtable* this, int key, void* value) {
 }
 
 void* Hashtable_remove(Hashtable* this, int key) {
-   int index = this->hashAlgorithm(this, key);
+   int index = key % this->size;
    HashtableItem** bucketPtr = &(this->buckets[index]);
    while (true)
       if (*bucketPtr == NULL) {
@@ -116,17 +109,20 @@ void* Hashtable_remove(Hashtable* this, int key) {
       } else
          bucketPtr = &((*bucketPtr)->next);
 }
-
+//#include <stdio.h>
 inline void* Hashtable_get(Hashtable* this, int key) {
-   int index = this->hashAlgorithm(this, key);
+   int index = key % this->size;
    HashtableItem* bucketPtr = this->buckets[index];
-   while (true)
+   // fprintf(stderr, "%d -> %d\n", key, index);
+   while (true) {
       if (bucketPtr == NULL) {
          return NULL;
       } else if (bucketPtr->key == key) {
          return bucketPtr->value;
       } else
          bucketPtr = bucketPtr->next;
+      // fprintf(stderr, "*\n");
+   }
 }
 
 void Hashtable_foreach(Hashtable* this, Hashtable_PairFunction f, void* userData) {
